@@ -122,7 +122,7 @@ static const struct menu menu_generic = {
 		MENU_BENT ('q', N_("quit without saving changes")),
 		MENU_XENT ('r', N_("return to main menu")),
 
-		MENU_ENT_NEST('r', N_("return from BSD to DOS"), FDISK_DISKLABEL_BSD, FDISK_DISKLABEL_DOS),
+		MENU_ENT_NEST('r', N_("return from BSD to DOS (MBR)"), FDISK_DISKLABEL_BSD, FDISK_DISKLABEL_DOS),
 
 		MENU_ENT_NEST('r', N_("return from protective/hybrid MBR to GPT"), FDISK_DISKLABEL_DOS, FDISK_DISKLABEL_GPT),
 
@@ -138,7 +138,7 @@ static const struct menu menu_createlabel = {
 		MENU_SEP(N_("Create a new label")),
 		MENU_ENT('g', N_("create a new empty GPT partition table")),
 		MENU_ENT('G', N_("create a new empty SGI (IRIX) partition table")),
-		MENU_ENT('o', N_("create a new empty DOS partition table")),
+		MENU_ENT('o', N_("create a new empty MBR (DOS) partition table")),
 		MENU_ENT('s', N_("create a new empty Sun partition table")),
 
 		/* backward compatibility -- be sensitive to 'g', but don't
@@ -221,6 +221,7 @@ static const struct menu menu_dos = {
 		MENU_ENT('c', N_("toggle the dos compatibility flag")),
 
 		MENU_XENT('b', N_("move beginning of data in a partition")),
+		MENU_XENT('F', N_("fix partitions C/H/S values")),
 		MENU_XENT('i', N_("change the disk identifier")),
 
 		MENU_BENT_NEST_H('M', N_("return from protective/hybrid MBR to GPT"), FDISK_DISKLABEL_DOS, FDISK_DISKLABEL_GPT),
@@ -432,8 +433,10 @@ int process_fdisk_menu(struct fdisk_context **cxt0)
 		/* Map ^C and ^D in main menu to 'q' */
 		if (is_interactive
 		    && fdisk_label_is_changed(fdisk_get_label(cxt, NULL))) {
+			/* TRANSLATORS: these yes no questions use rpmatch(),
+			 * and should be translated.  */
 			rc = get_user_reply(
-				_("\nAll unwritten changes will be lost, do you really want to quit? "),
+				_("\nAll unwritten changes will be lost, do you really want to quit? (y/n)"),
 				buf, sizeof(buf));
 			if (rc || !rpmatch(buf))
 				return 0;
@@ -869,6 +872,13 @@ static int dos_menu_cb(struct fdisk_context **cxt0,
 			fdisk_unref_context(cxt);
 		}
 		break;
+	case 'F':
+		rc = fdisk_dos_fix_chs(cxt);
+		if (rc)
+			fdisk_info(cxt, _("C/H/S values fixed."));
+		else
+			fdisk_info(cxt, _("Nothing to do. C/H/S values are correct already."));
+		break;
 	}
 	return rc;
 }
