@@ -1,4 +1,6 @@
 /*
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * Copyright (C) 2014 Ondrej Oprala <ooprala@redhat.com>
  * Copyright (C) 2014 Karel Zak <kzak@redhat.com>
  *
@@ -49,11 +51,15 @@ struct ul_debug_maskname {
 #define UL_DEBUG_DEFINE_MASKNAMES(m) static const struct ul_debug_maskname m ## _masknames[]
 #define UL_DEBUG_MASKNAMES(m)	m ## _masknames
 
-#define UL_DEBUG_DEFINE_MASK(m) int m ## _debug_mask
+#define UL_DEBUG_MASK(m)         m ## _debug_mask
+#define UL_DEBUG_DEFINE_MASK(m)  int UL_DEBUG_MASK(m)
 #define UL_DEBUG_DECLARE_MASK(m) extern UL_DEBUG_DEFINE_MASK(m)
 
-/* p - flag prefix, m - flag postfix */
-#define UL_DEBUG_DEFINE_FLAG(p, m) p ## m
+/*
+ * Internal mask flags (above 0xffffff)
+ */
+#define __UL_DEBUG_FL_NOADDR	(1 << 24)	/* Don't print object address */
+
 
 /* l - library name, p - flag prefix, m - flag postfix, x - function */
 #define __UL_DBG(l, p, m, x) \
@@ -79,21 +85,32 @@ struct ul_debug_maskname {
 		} \
 	} while (0)
 
-
-#define __UL_INIT_DEBUG(lib, pref, mask, env) \
+#define __UL_INIT_DEBUG_FROM_STRING(lib, pref, mask, str) \
 	do { \
 		if (lib ## _debug_mask & pref ## INIT) \
 		; \
-		else if (!mask) { \
-			char *str = getenv(# env); \
-			if (str) \
-				lib ## _debug_mask = ul_debug_parse_envmask(lib ## _masknames, str); \
+		else if (!mask && str) { \
+			lib ## _debug_mask = ul_debug_parse_mask(lib ## _masknames, str); \
 		} else \
 			lib ## _debug_mask = mask; \
+		if (lib ## _debug_mask) { \
+			if (getuid() != geteuid() || getgid() != getegid()) { \
+				lib ## _debug_mask |= __UL_DEBUG_FL_NOADDR; \
+				fprintf(stderr, "%d: %s: don't print memory addresses (SUID executable).\n", getpid(), # lib); \
+			} \
+		} \
 		lib ## _debug_mask |= pref ## INIT; \
 	} while (0)
 
 
+#define __UL_INIT_DEBUG_FROM_ENV(lib, pref, mask, env) \
+	do { \
+		const char *envstr = mask ? NULL : getenv(# env); \
+		__UL_INIT_DEBUG_FROM_STRING(lib, pref, mask, envstr); \
+	} while (0)
+
+
+
 static inline void __attribute__ ((__format__ (__printf__, 1, 2)))
 ul_debug(const char *mesg, ...)
 {
@@ -104,20 +121,7 @@ ul_debug(const char *mesg, ...)
 	fputc('\n', stderr);
 }
 
-static inline void __attribute__ ((__format__ (__printf__, 2, 3)))
-ul_debugobj(const void *handler, const char *mesg, ...)
-{
-	va_list ap;
-
-	if (handler)
-		fprintf(stderr, "[%p]: ", handler);
-	va_start(ap, mesg);
-	vfprintf(stderr, mesg, ap);
-	va_end(ap);
-	fputc('\n', stderr);
-}
-
-static inline int ul_debug_parse_envmask(
+static inline int ul_debug_parse_mask(
 			const struct ul_debug_maskname flagnames[],
 			const char *mask)
 {
@@ -171,7 +175,7 @@ static inline void ul_debug_print_masks(
 	for (d = flagnames; d && d->name; d++) {
 		if (!d->help)
 			continue;
-		fprintf(stderr, "   %-8s [0x%04x] : %s\n",
+		fprintf(stderr, "   %-8s [0x%06x] : %s\n",
 				d->name, d->mask, d->help);
 	}
 }
