@@ -1,4 +1,6 @@
 /*
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * Very simple multibyte buffer editor. Allows to maintaine the current
  * position in the string, add and remove chars on the current position.
  *
@@ -151,17 +153,20 @@ static size_t mbs_insert(char *str, wint_t c, size_t *ncells)
 {
 	/* all in bytes! */
 	size_t n = 1, bytes;
-	char *in = (char *) &c;
+	char *in;
 
 #ifdef HAVE_WIDECHAR
 	wchar_t wc = (wchar_t) c;
 	char in_buf[MB_CUR_MAX];
 
 	n = wctomb(in_buf, wc);
+	if (n == (size_t) -1)
+		return n;
 	*ncells = wcwidth(wc);
 	in = in_buf;
 #else
 	*ncells = 1;
+	in = (char *) &c;
 #endif
 	bytes       = strlen(str);
 
