@@ -5,7 +5,7 @@
 
 extern struct libmnt_cache *mntcache;
 
-extern struct libmnt_table *get_fstab(void);
+extern struct libmnt_table *get_fstab(const char *filename);
 extern struct libmnt_table *get_swaps(void);
 extern void free_tables(void);
 
@@ -22,5 +22,4 @@ extern void add_uuid(const char *uuid);
 extern const char *get_uuid(size_t i);
 extern size_t numof_uuids(void);
 
-
 #endif /* UTIL_LINUX_SWAPON_COMMON_H */
